
/*
 * TalkServer.java
 * The Server for the chat room program.
 * Receives commands from the client,
 * Executes the commands and
 * sends output back to the client
 *
 * Created on September 15, 2003, 11:40 PM
 */

/**
 *
 * @author  Nathan Balon
 * SN 3210 1717
 * CIS 527
 * Program Assignment #1
 * 
 */


import java.io.*;
import java.net.*;
import java.util.*;

/** TalkServer implements the Server portion of the chat program */
public class TalkServer {

    // port # of the chat server 
    private final static short SERVER_PORT = 1717;    
    
    /** Creates a new instance of talkServer */
    public TalkServer() {
        System.out.println("TalkServer Started on port #"
                           + SERVER_PORT);
    }
    

    public static void main(String args[]){      
        UserMap userMap = new UserMap();        // user file from disk
        HashMap loggedInUsers = new HashMap();  // logged in users
        ServerSocket serverSocket = null;       // server socket
        Socket clientSocket = null;             // client socket
        
        // open the userList file with users and passwords
        try{
            userMap = (UserMap)userMap.open();
        }

        // if the file does not yet exist create the default file
        catch (FileNotFoundException e){
            userMap.createDefaultUsers();
        }

        // catch other exceptions due to open the userList file
        catch (Exception e){
            System.err.println("Unexpected ERROR: " + e);
        }

   
        try{    
           TalkServer talkServer = new TalkServer();
           // create a server socket to listen for incoming connection
           serverSocket = new ServerSocket(SERVER_PORT);
           while((clientSocket = serverSocket.accept()) != null){
   
              // create a BufferedReader to read input streams from the client.
              BufferedReader is = new BufferedReader
                   (new InputStreamReader(clientSocket.getInputStream()));
              //create a PrintWriter to write output streams to the client.
              PrintWriter os = new PrintWriter
                (clientSocket.getOutputStream(), true);
           
              String inputStream = "";       // input recieved from the client
              String clientCommand = "";     // command to execute
           
              // loop to read strings from the client 
              // and process the correct commands.
              while((inputStream = is.readLine()) != null){ 

                   StringTokenizer tok = new StringTokenizer(inputStream);
                   // get the clients command
                   String clientInput = tok.nextToken();  
                   // convert the command to lower case
                   clientInput = clientInput.toLowerCase();
                
                   // process the send command
                   if(clientInput.equals("send")){
                       // display users messageon server
                       System.out.println("SERVER# " +
                                          inputStream);
                     
                       String message = "SERVER# ";
                       // get the message to send
                       while(tok.hasMoreTokens()){
                            message += tok.nextToken() + " ";
                       }
                       // send message to the client
                       os.println(message);    
                   }
                   else if(clientInput.equals("new")){
                       String name = " ";      // new users name
                       String pass = " ";      // new user password
                    
                       // get the new users name and pass
                       name = tok.nextToken();
                       pass = tok.nextToken();
                    
                       // check if the user already exist
                       if(userMap.containsKey(name)){
                           os.println("SERVER# user already " +
                                      "has an account");
                       }
                       // add the user to the file
                       else{
                           // create a new user
                           User newUser = new User(name, pass);   
                           // add the user to the list of users
                           userMap.put(name, newUser);
                           // save the user to the file
                           userMap.save();
                           // display sucess message
                           System.out.println("SERVER# Added new user "
                                              + name);
                           os.println("SERVER# Added new user " + name );    
                       }
                   }
                
                   else if(clientInput.equals("login")){
                       String name = "";             // user name
                       String pass = "";             // user password
                    
                       // get the user name and password
                       name = tok.nextToken();       
                       pass = tok.nextToken();       

                       // check if user name exist in the user list
                       if(userMap.containsKey(name)){
                           // if name exists get the user
                           User u = (User)userMap.get(name);
                           // if the password match the user password
                           // log them in to chat
                           if(pass.equals(u.getPassword())){
                            System.out.println("SERVER# " + name + 
                                               " joined the chat room"); 
                            // return message that user is logged in
                            os.println("true " + u.getName() + " " + 
                                       u.getPassword() + " SERVER# "
                                       + name + 
                                       " joined the chat room");
                            // put user in list of logged in users
                            loggedInUsers.put(u.getName(), u);                         
                        }
                        else{
                            // if name is in list but passwords don't match
                            System.out.println("SERVER# User " + name + 
                                               " incorrect password"); 
                            os.println("false SERVER# User " + name + 
                                       " incorrect password");   
                        }  
                    }
                    else{
                        // if the user can't be logged in because 
                        // they have registered
                        os.println("false SERVER# User not found " +
                                   "please register");
                        System.out.println("SERVER# User not found " +
                                           "please register");
                    }
                }
                // log the user out of the chat room
                else if(clientInput.equals("logout")){
                    String name = tok.nextToken();
                    String pass = tok.nextToken();
                    // rmove user from list of logged in users
                    loggedInUsers.remove(name);
                    System.out.println("SERVER# " + name +
                                       " logged out of chat");
                }
                // if user terminate chat
                else if(clientInput.equals("quit")){
                    System.out.println("SERVER# user quit chat");
                    // close the socket to the client
                   //  clientSocket.close();
                }
                else{
                   System.out.println("SERVER# Unknown Command was entered");
                }
             
              }
              if(clientSocket != null){
                  clientSocket.close();
              }
          }
           
       }
       catch (Exception e){
           e.printStackTrace();
       }
       finally{
           try{  
               if(clientSocket != null)
                   clientSocket.close();
               if(serverSocket != null)
                   serverSocket.close();
           }
           catch (IOException e){
               System.err.println("IO Error: " + e);
           }
       }
    }   
}

